"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types = require("babel-types");
var nejmap_1 = require("nejm/nejmap");
var util_1 = require("./util");
/**
 * 分析nej模块的依赖
 *
 * @param source 依赖路径
 *
 * NEJ 依赖举例
 * 1. base/util                 导入 nej 模块
 * 2. pro/extend/util
 *    {pro}/extend/util.js      导入 pro 文件夹下的模块
 * 3. text!/path/to/file.css    导入文本文件
 * 4.
 */
var analyisDeps = function (source) {
    var nejmName = nejmap_1.source2name[source];
    var isNej = nejmName !== undefined;
    var isText = util_1.TEXT_DEPS.test(source);
    return {
        source: source,
        isNej: isNej,
        isText: isText,
        nejmName: nejmName
    };
};
var defaultOptions = {
    replaceTextModule: true,
    extName: '',
    alias: {}
};
/**
 * 获取nej模块依赖
 *
 * @param path
 */
function default_1(path, options) {
    if (options === void 0) { options = {}; }
    options = Object.assign(defaultOptions, options);
    var fnBody;
    var deps = [];
    var custormModule = [];
    var textModule = [];
    var nejModule = [];
    var nejInject = [];
    var aliasRe = util_1.genAliasRe(options.alias);
    var visitor = {
        CallExpression: function (_a) {
            var node = _a.node;
            if (node.callee.name === 'define') {
                var _b = node.arguments, depsArray = _b[0], fnExpression = _b[1];
                if (types.isFunctionExpression(depsArray)) {
                    fnExpression = depsArray;
                    depsArray = {
                        elements: []
                    };
                }
                depsArray.elements.forEach(function (_a) {
                    var value = _a.value;
                    deps.push(analyisDeps(value));
                });
                fnBody = fnExpression.body.body;
                fnExpression.params.forEach(function (param, i) {
                    if (deps[i]) {
                        deps[i]['name'] = param.name;
                    }
                    else {
                        nejInject.push(param.name);
                    }
                });
            }
        }
    };
    path.traverse(visitor);
    deps.forEach(function (dep) {
        if (dep.isNej) {
            nejModule.push(dep);
        }
        else if (dep.isText) {
            if (options.replaceTextModule) {
                dep.source = dep.source.replace(util_1.TEXT_DEPS, '');
            }
            textModule.push(dep);
        }
        else {
            var result = dep.source.match(aliasRe); // 处理nej alias
            if (result) {
                dep.source = dep.source.replace(result[0], options.alias[result[0]]);
            }
            // 处理尾缀
            dep.source = dep.source.replace(/\.js/, options.extName);
            custormModule.push(dep);
        }
    });
    return {
        custormModule: custormModule,
        textModule: textModule,
        nejModule: nejModule,
        nejInject: nejInject,
        rawDeps: deps,
        fnBody: fnBody
    };
}
exports.default = default_1;
//# sourceMappingURL=transform-nej-es2015.js.map